/***********************************************************************************************************************
 *
 * blueMarine Semantic - open source media workflow
 * Copyright (C) 2008-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~semantic-src
 *
 **********************************************************************************************************************/
package it.tidalwave.openrdf.elmo;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

import it.tidalwave.util.As;
import it.tidalwave.util.Finder;
import it.tidalwave.semantic.Wrapper;
import javax.annotation.Nonnull;
import javax.xml.namespace.QName;
import org.junit.Test;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.CoreMatchers.instanceOf;
import static org.hamcrest.CoreMatchers.notNullValue;
import org.openrdf.elmo.ElmoQuery;
import org.openrdf.elmo.Entity;

/**
 *
 * @author fritz
 */
interface MockConcept
  {
  }

class MockConceptImpl implements MockConcept
  {
  }

public class ElmoOWLThingTest 
  {
    @Test
    public void shouldGenerateCorrectSPARQLForFindSameAsOfType()
      {
        Wrapper.register(MockConcept.class, MockConceptImpl.class);
        
        final QName id = new QName("urn:THEID");
        final Entity entity = mock(Entity.class);
        when(entity.getQName()).thenReturn(id);
        
        final As as = mock(As.class);
        when(as.as(eq(Entity.class))).thenReturn(entity);
        
        ElmoFinder.queryFactory = mock(ElmoFinder.QueryFactory.class);

        when(ElmoFinder.queryFactory.createQuery(anyString())).thenAnswer(new Answer<ElmoQuery>()
          {
            @Override
            public ElmoQuery answer (final @Nonnull InvocationOnMock invocation)
              {
                final ElmoQuery elmoQuery = mock(ElmoQuery.class);
                when(elmoQuery.setType(anyString(), any(Class.class))).thenReturn(elmoQuery);
                when(elmoQuery.setQName(anyString(), any(QName.class))).thenReturn(elmoQuery);
                        
                assertThat((String)invocation.getArguments()[0], is(
                        "PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>\n" +
                        "PREFIX owl: <http://www.w3.org/2002/07/owl#>\n" +
                        "SELECT ?mockConcept\n" +
                        "WHERE\n" +
                        "  {\n" +
                        "    ?mockConcept a ?type.\n" +
                        "    { ?id owl:sameAs ?mockConcept } UNION { ?mockConcept owl:sameAs ?id }\n" +
                        "  }\n"));
                return elmoQuery;
              }
          });

        final ElmoOwlThing fixture = new ElmoOwlThing(as);
        final Finder<MockConcept> finder = fixture.findSameAs().ofType(MockConcept.class);

        assertThat(finder, is(notNullValue()));
        assertThat(finder, is(instanceOf(ElmoFinder.class)));
        final ElmoFinder<MockConcept> elmoFinder = (ElmoFinder<MockConcept>)finder;
        final ElmoQuery elmoQuery = elmoFinder.createQuery();
        assertThat(elmoQuery, is(notNullValue()));
        verify(elmoQuery).setType(eq("type"), eq(MockConceptImpl.class));
        verify(elmoQuery).setQName(eq("id"), same(id));
      }
  }